@@ -39,6 +39,12 @@ class MemberDict(dict):
             d[key] /= b
         return d
 
+    def __getstate__(self):
+        return dict(self)
+
+    def __setstate__(self, state):
+        self.__dict__ = state
+
 class ArgumentDescriptor(object):
     """The base descriptor that wraps arguments of `add_argument` for `argparse.ArgumentParser`.
     """
@@ -223,7 +229,7 @@ class HeavenArguments(MemberDict):
         Args:
             descriptors (List): The list of argument descriptors.
             parser_args: Arguments for constructing `argparse.ArgumentParser`.
-        Return:
+        Returns:
             Arguments: The parsed arguments.
         """
         parser = argparse.ArgumentParser(**parser_args)
@@ -238,7 +244,7 @@ class HeavenArguments(MemberDict):
         Args:
             path: The json file path.
             backend (str): Specify backend for saving an object in str format. Please refer to function `BUILTIN_JSON_BACKENDS()` for built-in backends.
-        Return:
+        Returns:
             Arguments: The parsed arguments.
         """
         return cls(LoadJson(path,backend=backend))
@@ -249,7 +255,7 @@ class HeavenArguments(MemberDict):
 
         Args:
             obj: The object.
-        Return:
+        Returns:
             Arguments: The parsed arguments.
         """
         return cls(vars(object))
@@ -264,7 +270,7 @@ class HeavenArguments(MemberDict):
                           If 'interupt', an Exception will be raised immediately at the error line.
                           If 'strict', a ValueError will be raised to report all error lines.
                           If 'ignore', all error lines will be skipped.
-        Return:
+        Returns:
             Arguments: The parsed arguments.
         """
         args = {}; failures = []
